--------------------
-- SkyShards v0.3 --
--------------------
if SkyS == nil then SkyS = {} end

-- default settings for saved variables
SkyS.defaults = {
	showUnknown = true,
	showCollected = true,
	showCompassUnknown = true,
	pinTextureType = 1,
}

SkyS.language = ""

-- helper functions
function SkyS.Print(message)
	CHAT_SYSTEM:AddMessage(message)
end

local function GetZoneAndSubzone()
	local zone = ""
	local subzone = ""
	local textureName = GetMapTileTexture()
	
	textureName = string.lower(textureName)
	textureName = string.gsub(textureName, "^.*maps/", "")
	textureName = string.gsub(textureName, "_%d+%.dds$", "")
	
	zone = string.gsub(textureName, "/.*$", "")
	subzone = string.gsub(textureName, "^.*/", "")

   return zone, subzone
end

-- Pins
SkyS.pinTextureType = 1

SkyS.pinTexturesList = {
	"default icons (@Garkin)",
	"UESP's icons (@Garkin)",
	"Esohead's icons (Mitsarugi)"	
}

SkyS.pinTextures = {
	unknown = {
		[1] = "SkyShards/Icons/SkyShard-unknown.dds", 
		[2] = "SkyShards/Icons/SkyShard-unknown-UESP.dds", 
		[3] = "SkyShards/Icons/SkyShard-unknown-Esohead.dds", 
	},
	collected = {
		[1] = "SkyShards/Icons/SkyShard-collected.dds", 
		[2] = "SkyShards/Icons/SkyShard-collected-UESP.dds",
		[3] = "SkyShards/Icons/SkyShard-collected-Esohead.dds",
	},
}

SkyS.pinLayout_unknown = { 
	level = 40,
	texture = SkyS.pinTextures.unknown[SkyS.pinTextureType],
	size = 32,
}

SkyS.pinLayout_collected = {
	level = 40,
	texture = SkyS.pinTextures.collected[SkyS.pinTextureType],
	size = 32,
}

local function GetInfoFromTag(pin)
	local _, pinTag = pin:GetPinTypeAndTag()
	local name = GetAchievementInfo(pinTag[3]) 
	local description = "(" .. pinTag[4] ..") " .. GetAchievementCriterion(pinTag[3], pinTag[4])

	if pinTag[5] ~= nil then
		local info = "[" .. SkyS[SkyS.language].moreInfo[pinTag[5]] .. "]" 
		return name, description, info
	else
		return name, description
	end
end

SkyS.pinTooltipCreator_unknown = {
	creator = function(pin)
		local name, description, info = GetInfoFromTag(pin)
		InformationTooltip:AddLine(name, "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
		InformationTooltip:AddLine(description, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		if info ~= nil then
			InformationTooltip:AddLine(info, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		end
	end,
	tooltip = InformationTooltip,
}

SkyS.pinTooltipCreator_collected = {
	creator = function(pin)
		local name, description, info = GetInfoFromTag(pin)
		InformationTooltip:AddLine(name, "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
		InformationTooltip:AddLine(description, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		if info ~= nil then
			InformationTooltip:AddLine(info .. " / [" .. SkyS[SkyS.language].collected .. "]", "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		else
			InformationTooltip:AddLine("[" .. SkyS[SkyS.language].collected .. "]", "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		end
	end,
	tooltip = InformationTooltip,
}

function SkyS.AddCallback_unknown(pinManager)
	if not SkyS.SavedVariables.showUnknown then return end

	local zone, subzone = GetZoneAndSubzone()
	
	if SkyS.skyshardsData[zone] == nil or SkyS.skyshardsData[zone][subzone] == nil then
		return
	end

	local skyshards = SkyS.skyshardsData[zone][subzone]

	for _, pinData in pairs(skyshards) do
		local _, completed = GetAchievementCriterion(pinData[3], pinData[4])
		if completed == 0 then
			pinManager:CreatePin( _G["SkySMapPin_unknown"], pinData, pinData[1], pinData[2])
		end
	end
end

function SkyS.AddCallback_collected(pinManager)
	if not SkyS.SavedVariables.showCollected then return end

	local zone, subzone = GetZoneAndSubzone()
	
	if SkyS.skyshardsData[zone] == nil or SkyS.skyshardsData[zone][subzone] == nil then
		return
	end

	local skyshards = SkyS.skyshardsData[zone][subzone]
	
	if not skyshards then return end

	for _, pinData in pairs(skyshards) do
		local _, completed = GetAchievementCriterion(pinData[3], pinData[4])
		if completed == 1 then
			pinManager:CreatePin( _G["SkySMapPin_collected"], pinData, pinData[1], pinData[2])
		end
	end
end

function SkyS.CompassPinCallback_unknown(pinManager)
	if not SkyS.SavedVariables.showCompassUnknown then return end

	local zone, subzone = GetZoneAndSubzone()
	
	if SkyS.skyshardsData[zone] == nil or SkyS.skyshardsData[zone][subzone] == nil then
		return
	end

	local skyshards = SkyS.skyshardsData[zone][subzone]
	
	if not skyshards then return end

	for _, pinData in pairs(skyshards) do
		local _, completed = GetAchievementCriterion(pinData[3], pinData[4])
		if completed == 0 then
			pinManager:CreatePin( "SkySCompassPin_unknown" , pinData[1], pinData[2])
		end
	end
end

-- slash commands
function SkyS.ShowMyPosition()
	local setMap = SetMapToPlayerLocation()
	if setMap == 2 then
		CALLBACK_MANAGER:FireCallbacks("OnWorldMapChanged")
	end
	
	local x, y = GetMapPlayerPosition("player")
	local subzone = GetMapName()

	local locX = string.format("%03d", zo_round(x*1000))
	local locY = string.format("%03d", zo_round(y*1000))

	SkyS.Print(subzone .. ": " .. locX .. "," .. locY)
end

-- event handlers
function SkyS.OnAchievementUpdate(eventCode, achievementId)
	local ids = SkyS.achievementIDs

	for i = 1, #ids do
		if (ids[i] == achievementId) then
			SkyS.MapPins:RefreshPins()
			COMPASS_PINS:RefreshPins()
			break
		end
	end
end

function SkyS.OnZoneUpdate(...)
	COMPASS_PINS:RefreshPins()
end

function SkyS.OnLoad(event, name)
	if name ~= "SkyShards" then return end

	SkyS.SavedVariables = ZO_SavedVars:New("SkyS_SavedVariables", 3, nil, SkyS.defaults)

	SkyS.language = GetCVar("language.2") or "en"
	if SkyS.language ~= "en" and SkyS.language ~= "de" and SkyS.language ~= "fr" then
		SkyS.Print("Language \"" .. SkyS.language .. "\" is not supported. SkyShars will use default english localizaton.")
      SkyS.language = "en"
   end

	--get pin texture from saved variables
	SkyS.pinTextureType = SkyS.SavedVariables.pinTextureType or 1
	SkyS.pinLayout_unknown.texture = SkyS.pinTextures.unknown[SkyS.pinTextureType]
	SkyS.pinLayout_collected.texture = SkyS.pinTextures.collected[SkyS.pinTextureType]

	--initialize map pins
	SkyS.MapPins = CustomMapPins:New()
	SkyS.MapPins:CreatePinType("SkySMapPin_unknown", SkyS.pinLayout_unknown, SkyS.pinTooltipCreator_unknown, SkyS.AddCallback_unknown)
	SkyS.MapPins:CreatePinType("SkySMapPin_collected", SkyS.pinLayout_collected, SkyS.pinTooltipCreator_collected, SkyS.AddCallback_collected)
	SkyS.MapPins:RefreshPins()
	
	--initialize compass pins
	COMPASS_PINS:AddCustomPin("SkySCompassPin_unknown", SkyS.CompassPinCallback_unknown, SkyS.pinLayout_unknown)
	COMPASS_PINS:RefreshPins()


	-- addon menu
	local LAM = LibStub:GetLibrary("LibAddonMenu-1.0")
	local addonMenu = LAM:CreateControlPanel("SkyShards_OptionsPanel", "SkyShards")

	local function GetUnknown()
		return SkyS.SavedVariables.showUnknown
	end

	local function SwitchUnknown()
		SkyS.SavedVariables.showUnknown = not SkyS.SavedVariables.showUnknown
		SkyS.MapPins:RefreshPins()
	end

	local function GetCollected()
		return SkyS.SavedVariables.showCollected
	end

	local function SwitchCollected()
		SkyS.SavedVariables.showCollected = not SkyS.SavedVariables.showCollected
		SkyS.MapPins:RefreshPins()
	end
	
	local function GetCompassUnknown()
		return SkyS.SavedVariables.showCompassUnknown
	end

	local function SwitchCompassUnknown()
		SkyS.SavedVariables.showCompassUnknown = not SkyS.SavedVariables.showCompassUnknown
		COMPASS_PINS:RefreshPins()
	end

	local function GetCollected()
		return SkyS.SavedVariables.showCollected
	end

	local function GetPinTexture()
		return SkyS.pinTexturesList[SkyS.SavedVariables.pinTextureType]
	end

	local function SetPinTexture(pinTextureListName)
		for index, name in pairs(SkyS.pinTexturesList) do
			if name == pinTextureListName then
				SkyS.SavedVariables.pinTextureType = index
			end
		end
		ReloadUI("ingame")
	end

	LAM:AddHeader(addonMenu, "SkyShards_OptionsHeader", SkyS[SkyS.language].menuOptions)
	LAM:AddCheckbox(addonMenu, "SkyShards_showUnknow", SkyS[SkyS.language].menuUnknown, SkyS[SkyS.language].menuUnknownDesc, GetUnknown, SwitchUnknown)
	LAM:AddCheckbox(addonMenu, "SkyShards_showCollected", SkyS[SkyS.language].menuCollected, SkyS[SkyS.language].menuCollectedDesc, GetCollected, SwitchCollected)
	LAM:AddCheckbox(addonMenu, "SkyShards_showCompassUnknown", SkyS[SkyS.language].menuCompassUnknown, SkyS[SkyS.language].menuCompassUnknownDesc, GetCompassUnknown, SwitchCompassUnknown)
	LAM:AddDropdown(addonMenu, "SkyShards_pinTexture", SkyS[SkyS.language].menuPinTexture, SkyS[SkyS.language].menuPinTextureDesc, SkyS.pinTexturesList, GetPinTexture, SetPinTexture, true, SkyS[SkyS.language].menuPinTextureWarning)

	-- slash commands
	SLASH_COMMANDS["/mypos"] = SkyS.ShowMyPosition
	SLASH_COMMANDS["/myposition"] = SkyS.ShowMyPosition
	SLASH_COMMANDS["/myloc"] = SkyS.ShowMyPosition
	SLASH_COMMANDS["/mylocation"] = SkyS.ShowMyPosition


	EVENT_MANAGER:UnregisterForEvent("SkyShards", EVENT_ADD_ON_LOADED)
end

EVENT_MANAGER:RegisterForEvent("SkyShards_OnLoad", EVENT_ADD_ON_LOADED, SkyS.OnLoad)
EVENT_MANAGER:RegisterForEvent("SkyShards_OnAchievementUpdate",  EVENT_ACHIEVEMENT_UPDATED, SkyS.OnAchievementUpdate)
EVENT_MANAGER:RegisterForEvent("SkyShards_OnZoneChange", EVENT_ZONE_CHANGED, SkyS.OnZoneUpdate)
